@@ -64,6 +64,8 @@ from gevent._compat import PY35
 from gevent._compat import PY36
 from gevent._compat import PY37
 from gevent._compat import PY38
+from gevent._compat import PY311
+from gevent._compat import PYPY
 from gevent._compat import reraise
 from gevent._compat import fsdecode
 from gevent._compat import fsencode
@@ -209,6 +211,22 @@ if PY38:
             '_USE_POSIX_SPAWN',
         ])
 
+if PY311:
+    # Python 3.11 added some module-level attributes to control the
+    # use of vfork. The docs specifically say that you should not try to read
+    # them, only set them, so we don't provide them.
+    #
+    # Python 3.11 also added a test,  test_surrogates_error_message, that behaves
+    # differently based on whether or not the pure python implementation of forking
+    # is in use, or the one written in C from _posixsubprocess. Obviously we don't call
+    # that, so we need to make us look like a pure python version; it checks that this attribute
+    # is none for that.
+    _fork_exec = None
+    __implements__.extend([
+        '_fork_exec',
+    ] if sys.platform != 'win32' else [
+    ])
+
 actually_imported = copy_globals(__subprocess__, globals(),
                                  only_names=__imports__,
                                  ignore_missing_names=True)
@@ -248,6 +266,7 @@ for _x in ('run', 'CompletedProcess', 'TimeoutExpired'):
         __all__.append(_x)
 
 
+
 mswindows = sys.platform == 'win32'
 if mswindows:
     import msvcrt # pylint: disable=import-error
@@ -279,7 +298,7 @@ else:
     from gevent.os import fork_and_watch
 
 try:
-    BrokenPipeError
+    BrokenPipeError # pylint:disable=used-before-assignment
 except NameError: # Python 2
     class BrokenPipeError(Exception):
         "Never raised, never caught."
@@ -375,10 +394,14 @@ def check_output(*popenargs, **kwargs):
     .. versionchanged:: 1.2a1
        The ``input`` keyword argument is now accepted on all supported
        versions of Python, not just Python 3
+    .. versionchanged:: 22.08.0
+       Passing the ``check`` keyword argument is forbidden, just as in Python 3.11.
     """
     timeout = kwargs.pop('timeout', None)
     if 'stdout' in kwargs:
         raise ValueError('stdout argument not allowed, it will be overridden.')
+    if 'check' in kwargs:
+        raise ValueError('check argument not allowed, it will be overridden.')
     if 'input' in kwargs:
         if 'stdin' in kwargs:
             raise ValueError('stdin and input arguments may not both be used.')
@@ -536,7 +559,7 @@ class _CommunicatingGreenlets(object):
             # Python 2 doesn't have a BrokenPipeError.
             if isinstance(ex, BrokenPipeError) and ex.errno is None:
                 ex.errno = errno.EPIPE
-            if ex.errno != errno.EPIPE and ex.errno != errno.EINVAL:
+            if ex.errno not in (errno.EPIPE, errno.EINVAL):
                 raise
         finally:
             try:
@@ -625,6 +648,13 @@ class Popen(object):
        it may make many system calls and/or trips around the event loop to accomplish this).
        See :issue:`1711`.
 
+    .. versionchanged:: 21.12.0
+       Added the ``pipesize`` argument for compatibility with Python 3.10.
+       This is ignored on all platforms.
+
+    .. versionchanged:: 22.08.0
+       Added the ``process_group`` and ``check`` arguments for compatibility with
+       Python 3.11.
     """
 
     if GenericAlias is not None:
@@ -651,6 +681,10 @@ class Popen(object):
                  # Added in 3.9
                  group=None, extra_groups=None, user=None,
                  umask=-1,
+                 # Added in 3.10, but ignored.
+                 pipesize=-1,
+                 # Added in 3.11
+                 process_group=None,
                  # gevent additions
                  threadpool=None):
 
@@ -813,7 +847,7 @@ class Popen(object):
                                 errread, errwrite,
                                 restore_signals,
                                 gid, gids, uid, umask,
-                                start_new_session)
+                                start_new_session, process_group)
         except:
             # Cleanup if the child failed starting.
             # (gevent: New in python3, but reported as gevent bug in #347.
@@ -1182,7 +1216,7 @@ class Popen(object):
                            errread, errwrite,
                            unused_restore_signals,
                            unused_gid, unused_gids, unused_uid, unused_umask,
-                           unused_start_new_session):
+                           unused_start_new_session, unused_process_group):
             """Execute program (MS Windows version)"""
             # pylint:disable=undefined-variable
             assert not pass_fds, "pass_fds not supported on Windows."
@@ -1276,6 +1310,33 @@ class Popen(object):
                     # kill children.
                     creationflags |= CREATE_NEW_CONSOLE
 
+            # PyPy 2.7 7.3.6 is now producing these errors. This
+            # happens automatically on Posix platforms, and is built
+            # in to the CreateProcess call on CPython 2 & 3. It's not
+            # clear why we don't pick it up for free from the
+            # CreateProcess call on PyPy. Currently we don't test PyPy3 on Windows,
+            # so we don't know for sure if it's built into CreateProcess there.
+            if PYPY:
+                def _check_nul(s, err_kind=(ValueError if PY3 else TypeError)):
+                    if not s:
+                        return
+                    nul = b'\0' if isinstance(s, bytes) else '\0'
+                    if nul in s:
+                        # PyPy 2 expects a TypeError; Python 3 raises ValueError always.
+                        raise err_kind("argument must be a string without NUL characters")
+                def _check_env():
+                    if not env:
+                        return
+                    for k, v in env.items():
+                        _check_nul(k)
+                        _check_nul(v)
+                        if '=' in k:
+                            raise ValueError("'=' not allowed in environment keys")
+
+                _check_nul(executable)
+                _check_nul(args)
+                _check_env()
+
             # Start the process
             try:
                 hp, ht, pid, tid = CreateProcess(executable, args,
@@ -1564,7 +1625,7 @@ class Popen(object):
                            errread, errwrite,
                            restore_signals,
                            gid, gids, uid, umask,
-                           start_new_session):
+                           start_new_session, process_group):
             """Execute program (POSIX version)"""
 
             if PY3 and isinstance(args, (str, bytes)):
@@ -1698,7 +1759,8 @@ class Popen(object):
                                 os.setregid(gid, gid)
                             if uid:
                                 os.setreuid(uid, uid)
-
+                            if process_group is not None:
+                                os.setpgid(0, process_group)
                             if preexec_fn:
                                 preexec_fn()
 
@@ -1982,13 +2044,14 @@ def run(*popenargs, **kwargs):
 
     return CompletedProcess(process.args, retcode, stdout, stderr)
 
-def _gevent_did_monkey_patch(*_args):
+def _gevent_did_monkey_patch(target_module, *_args, **_kwargs):
     # Beginning on 3.8 on Mac, the 'spawn' method became the default
     # start method. That doesn't fire fork watchers and we can't
     # easily patch to make it do so: multiprocessing uses the private
     # c accelerated _subprocess module to implement this. Instead we revert
     # back to using fork.
     from gevent._compat import MAC
+
     if MAC:
         import multiprocessing
         if hasattr(multiprocessing, 'set_start_method'):
